function [cell,angleChange,percentSmooth,ACR]=cellAngle_v1(stat,cell,FrameSeconds,...
                                TumbleSeconds,SmoothThreshold)


TumbleFrames=round(TumbleSeconds/FrameSeconds);     % Convert seconds to frames
SmoothThreshold=SmoothThreshold*FrameSeconds;       % Convert degrees per second to degrees per frame


%%% FINDING ANGLE CHANGE
angleChange=0;
percentSmooth=0;
ACR=0;

for u=1:numel(cell.real)                                % Analyzes one trajectory at a time
    
% FINDING BEGINNING AND END OF CLEAR REGIONS (NOT STUCK AT ALL)
    [clear,~]=find(~isnan(cell.real(u).track(:,1))); 	% Finds all non-NaN "clear" locations in cell.real
    first=clear(1);                                     % first holds row locations of beginning of clear regions
    last=0;                                             % last holds row locations of end of clear regions
    index=0;
    for row=1:length(clear)-1
        if clear(row+1)~=clear(row)+1                   %If next non-NaN location isn't consecutive, end of an clear region
            index=index+1;
            last(index)=clear(row);
            first(index+1)=clear(row+1);
        end
    end
    [last(end+1),~]=find(~isnan(cell.real(u).track),1,'last');      %Last non-NaN number
    if last(end-1)==last(end)
        last(end)=[];
        first(end)=[];
    end
    if last(1)==0
        last(1)=[];
    end
    
% FINDING ANGLE CHANGES, ACR, MEAN SQUARED DISPLACEMENT
    for row=1:length(first)
        if last(row)-first(row)<TumbleFrames   	% If traj not long enough, exclude
            continue
        end
                                             	% Euclidean Displacement
        TotalDisp=hypot(cell.real(u).track(first(row),1)-cell.real(u).track(first(row)+TumbleFrames,1),...
            cell.real(u).track(first(row),2)-cell.real(u).track(first(row)+TumbleFrames,2));
        Distance=zeros(1,TumbleFrames);
        
        SmoothTurns=0;
        changefree=zeros(1,TumbleFrames);                       % changefree holds angle change between each pair of frames
        for v=1:TumbleFrames
            lastspot=cell.real(u).index(first(row)-1+v,1);
            spot=cell.real(u).index(first(row)+v,1);
            change=abs(stat(lastspot).Orientation-stat(spot).Orientation);  % Find changes in cell orientation
            if change>90
                change=180-change;
            end
            if change<SmoothThreshold                           % Test if angle change is less than Smooth Threshold
                SmoothTurns=SmoothTurns+1;
            end
            changefree(v)=change;
                                                    % Find distance travelled between each pair of frames
            Distance(v)=hypot(cell.real(u).track(first(row)+v-1,1)-cell.real(u).track(first(row)+v,1),...
                cell.real(u).track(first(row)+v-1,2)-cell.real(u).track(first(row)+v,2));
            if isnan(Distance(v))
                Distance(v)=0;
            end
        end
        
% ADDING INDIVIDUAL RESULTS TO MASTER LIST OF RESULTS
        angleChange(end+1)=mean(changefree);
        percentSmooth(end+1)=SmoothTurns/TumbleFrames;
        ACR(end+1)=sum(Distance)/TotalDisp;
        break
    end
end

% TRIMMING RESULT LISTS
angleChange(1)=[];
angleChange=(angleChange/FrameSeconds);
percentSmooth(1)=[];
percentSmooth=percentSmooth*100;
ACR(1)=[];
ACR(ACR==Inf)=[];

if isempty(angleChange)
    error('TumbleSecs is too long. No trajectories are this length. Please lower value.');
end

