function [stat,Kindex]=cellDetect_v1(S,C,R,Kcount,TestFrame)
global PleaseStop       %Used in TestFrame GUI

%%% TEST FRAME
if TestFrame
    if TestFrame>Kcount
        error('Frame %d does not exist. Change TestFrame. \n',TestFrame);
    end
    if ~(mod(TestFrame,1)==0)
        error('TestFrame must be an integer.');
    end
    stat1=regionprops(S(:,:,TestFrame),'Centroid');    % Find all centroids in the TestFrame
    circleY=zeros(length(stat1),1);
    circleX=zeros(length(stat1),1);
    for u=1:length(stat1)
        circleY(u)=stat1(u).Centroid(2);
        circleX(u)=stat1(u).Centroid(1);
    end
    
% GUI FOR TEST FRAME
    nfigs=get(0,'Children');                   % Fixes glitch where MATLAB opens a blank figure
    title=sprintf('Motility Video - Test Frame (#%d)',TestFrame);
    PleaseStop=0;
    
    AspectRatio=size(S,1)/size(S,2);           % Must fit image into figure
    if AspectRatio<=0.75
        horiz=0.96;
        AxisRatio=[0.02 0.99-(horiz*AspectRatio) horiz (horiz*AspectRatio)];
    elseif AspectRatio>0.75
        horiz=0.72;
        AxisRatio=[0.5-(horiz/AspectRatio)/2 0.27 (horiz/AspectRatio) horiz];
    end
    
    scrsz=get(0,'ScreenSize');
    white=[1 1 1];
    hwindow=figure('Visible','on','Position',[scrsz(3)/4 scrsz(4)/4-150 scrsz(3)/2 5*scrsz(4)/6],...
    'Name',title,'Color',white,'MenuBar','none','KeyReleaseFcn',@Press_Callback);
    
    h=struct();
    h.ax=axes('Parent',hwindow,'Position',AxisRatio,'XColor',white,'YColor',white);

    if Kcount==size(C,3)
        imshow(C(:,:,TestFrame),'Parent',h.ax);
    else
        imshow(C(:,:,TestFrame*3-2:TestFrame*3),'Parent',h.ax);
    end
    hold on
    scatter(circleX,circleY,100,'MarkerEdgeColor','m','LineWidth',.2)
    hold off
    
    h.c.box = uipanel('Parent',hwindow,'Units','Normalized','Position',[0 0 1 0.25],'BackgroundColor',white);
    h.c.Stop=uicontrol('Parent',h.c.box,'Units','Normalized','Style','PushButton','BackgroundColor',white,...
    'Position',[0.1 0.15 0.3 0.75],'FontSize',20,'String','Stop');
    h.c.Continue=uicontrol('Parent',h.c.box,'Units','Normalized','Style','PushButton','BackgroundColor',white,...
    'Position',[0.6 0.15 0.3 0.75],'FontSize',20,'String','Continue (Hit Any Key)');
    
    set(h.c.Stop,'Callback',{@GUI_Callbacks,h});
    set(h.c.Continue,'Callback',{@GUI_Callbacks,h});
    
    uiwait
    
    if PleaseStop                       % If User signals that they will not proceed
        error('Cancelled By User')
    end
    if isempty(nfigs)
        close
    end
end


%%% DETECTION
stat=[];
Kindex=[];
for k=1:Kcount                                  % Detect all video parameters, store in stat. structure
    stat_new=regionprops(S(:,:,k),R(:,:,k),'Centroid','PixelList','Perimeter',...
                                        'Area','MajorAxisLength','Orientation','MaxIntensity');
    stat_new([stat_new.Area]<=1)=[];
    new=numel(stat_new);
    [stat(end+1:end+new).Centroid]=stat_new.Centroid;
    [stat(end-new+1:end).PixelList]=stat_new.PixelList;
    [stat(end-new+1:end).Perimeter]=stat_new.Perimeter;
    [stat(end-new+1:end).Area]=stat_new.Area;
    [stat(end-new+1:end).MajorAxisLength]=stat_new.MajorAxisLength;
    [stat(end-new+1:end).Orientation]=stat_new.Orientation;
    [stat(end-new+1:end).MaxIntensity]=stat_new.MaxIntensity;
    Kindex(k)=numel(stat);                      % Kindex keeps track of how many cells (in stat.) are detected in each frame
    
    if k>1                                      % Displays progress of detection
    fprintf(repmat('\b',1,n));
    end
    msg=sprintf('Detecting... Frame %d/%d.\n',k,Kcount);
    fprintf(msg);
    n=numel(msg);
end

index=1;
for u=1:numel(stat)                             % Trims detected cells
    if isempty(stat(u).PixelList)               % Discards any "cells" without at least one pixel (glitch)
        if any(Kindex==u)
            Kindex(Kindex==u)=index-1;
        end
    elseif ~(any(any(stat(u).PixelList==1)) ... % Keeps cells not partially out of frame
            || any(stat(u).PixelList(:,1)==size(S,2)) || any(stat(u).PixelList(:,2)==size(S,1)))
        stat(u).Centroid=fliplr(stat(u).Centroid);  % Centroid data is kept as (y,x) columns
        if stat(u).Orientation<0                % Cell orientation can only range from 0-180 degrees
            stat(u).Orientation=180+stat(u).Orientation;
        end
        stat(index)=stat(u);                    % Indexing
        if any(Kindex==u)
            Kindex(Kindex==u)=index;
        end
        index=index+1;
    else                                        % Discards cells that are partially out of frame
        if any(Kindex==u)
            Kindex(Kindex==u)=index-1;
        end
    end
end
stat(index:end)=[];
stat=rmfield(stat,'PixelList');                 % Discards unnecessary data


%%% GUI FUNCTION
function GUI_Callbacks(hObject,eventdata,h)
global PleaseStop
switch hObject
    case h.c.Stop
        close
        PleaseStop=1;
    case h.c.Continue
        close
        uiresume
end
function Press_Callback(hObject,callbackdata)
close
uiresume
    
    