function cell=cellTrack_v1(stat,Kcount,Kindex,NmPixel,DispCutoff,MinimumSeconds,FrameSeconds)

MinimumFrames=MinimumSeconds/FrameSeconds;              % Convert seconds to frames
DispCutoff=DispCutoff/(NmPixel/1000);                   % Convert nm to pixels
if DispCutoff<=0
    error('Please enter in a positive value for DispCutoff.');
end
if NmPixel<=0
    error('Please enter in a positive value for NmPixel.');
end

msg=sprintf('Tracking... Frame 1/%d. \n',Kcount);       % Displays progress of tracking
fprintf(msg);
n=numel(msg);

%%% LINKING CELLS
cell=struct();                                  % cell. structure holds all centroid data for each trajectory
for u=1:Kindex(1)                               % First frame.
cell.track(u).track(1,1:2)=stat(u).Centroid;            % .track contains all centroids (y,x) of cells
cell.track(u).index(1,1)=u;                             % .index(column 1) refers to cell's index in stat. structure
cell.track(u).index(1,2)=1;                             % .index(column 2) refers to the frame each cell is in
end

% FINDING DISTANCE BETWEEN OLD (TRAJ) AND NEW (SUITOR) CELLS
for k=2:Kcount                                  % Builds up trajectories frame by frame
    distance=zeros(Kindex(k)-Kindex(k-1),numel(cell.track));    % distance contains the distance between the last cell in each
    for u=1:numel(cell.track)                                   %  trajectory and every cell in the next frame (suitor)
        if isnan(cell.track(u).track(end,1))            % Does not calculate distances for deactivated trajectories
            distance(:,u)=Inf;
            continue
        end
        suitor=0;
        for v=Kindex(k-1)+1:Kindex(k)               % Tests each traj cell against each suitors
            suitor=suitor+1;
            distance(suitor,u)=hypot((cell.track(u).track(k-1,1)-stat(v).Centroid(1)),...
               (cell.track(u).track(k-1,2)-stat(v).Centroid(2)));
        end
    end
    
% FINDING OPTIMAL MATCH "NEAREST-NEIGHBOR" BETWEEN TRAJ AND SUITOR CELLS
    for v=1:size(distance,1)                        % v is suitor
        spot=Kindex(k-1)+v;                         % spot is index of suitor in stat. structure
        old=find(distance(v,:)<DispCutoff)';        % old contains all old traj cells within DispCutoff of suitor
        if isempty(old)                             % If no traj cells within range, suitor starts new trajectory
            cell.track(end+1).track(k,1:2)=stat(spot).Centroid;
            cell.track(end).index(k,1:2)=[spot,k];
            continue
        end
        old(:,2)=distance(v,old)';
        for OldTrajCell=1:size(old,1)                   % Find "nearness coefficient" to suitor
            AreaDiff=abs(stat(spot).Area-...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Area)/...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Area;
            PerimeterDiff=abs(stat(spot).Perimeter-...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Perimeter)/...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Perimeter;
            LengthDiff=abs(stat(spot).MajorAxisLength-...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MajorAxisLength)/...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MajorAxisLength;
            IntenseDiff=abs(stat(spot).MaxIntensity-...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MaxIntensity)/...
                stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MaxIntensity;
            old(OldTrajCell,3)=AreaDiff+PerimeterDiff+LengthDiff+IntenseDiff;
        end
        if size(old,1)>1                            % If traj cells are of similar nearness, chooses closer traj cell
            old=sortrows(old,3);
            closeness=abs(diff(old(:,3)));          % Sees how similar "nearness coefficients" are
            for i=1:length(closeness)
                if closeness(i)<0.1                 % If similar enough, closer cell wins
                    old(i,3)=old(i+1,3);
                end
            end
            old=sortrows(old,[3 2]);                % Sort traj cells by nearness to suitor
        end
        
        
        Matched=0;      % Compare other suitors against current suitor for each traj cell, in order of best matches
        for OldTrajCell=1:size(old,1)
            if any(distance(:,old(OldTrajCell,1))<DispCutoff)  	% Check if any other suitors are within range of traj cell
                othersuitors=find(distance(:,old(OldTrajCell,1))<DispCutoff);  	% Find such other suitors
                othersuitors(:,2)=distance(othersuitors,old(OldTrajCell,1));  	% Find distance from other suitors
                for p=1:size(othersuitors,1)                    % Find "nearness coefficient" to traj cell
                    AreaDiff=abs(stat(Kindex(k-1)+othersuitors(p,1)).Area-...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Area)/...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Area;
                    PerimeterDiff=abs(stat(Kindex(k-1)+othersuitors(p,1)).Perimeter-...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Perimeter)/...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).Perimeter;
                    LengthDiff=abs(stat(Kindex(k-1)+othersuitors(p,1)).MajorAxisLength-...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MajorAxisLength)/...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MajorAxisLength;
                    IntenseDiff=abs(stat(Kindex(k-1)+othersuitors(p,1)).MaxIntensity-...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MaxIntensity)/...
                        stat(cell.track(old(OldTrajCell,1)).index(k-1,1)).MaxIntensity;
                    othersuitors(p,3)=AreaDiff+PerimeterDiff+LengthDiff+IntenseDiff;
                end
                if size(othersuitors,1)>1       % If other suitors are of similar nearness, chooses closer other suitor
                    othersuitors=sortrows(othersuitors,3);
                    closeness=abs(diff(othersuitors(:,3)));
                    for i=1:length(closeness)
                        if closeness(i)<0.1
                            othersuitors(i,3)=othersuitors(i+1,3);
                        end
                    end
                    othersuitors=sortrows(othersuitors,[3 2]);
                end
                if othersuitors(1,3)<old(OldTrajCell,3) || ...
                        (othersuitors(1,3)==old(OldTrajCell,3) && othersuitors(1,2)<old(OldTrajCell,2))
                    continue    	% If another suitor matches the traj cell better, then we check the next traj cell
                else                % Else, this is the best match!
                    Matched=1;
                    break
                end
            else                    % If no other suitors are within range of traj cell, this is the best match!
                Matched=1;
                break
            end
        end
        if Matched                  % We matched, the suitor is added to the trajectory
            distance(:,old(OldTrajCell,1))=Inf;
            distance(v,:)=Inf;
            cell.track(old(OldTrajCell,1)).track(k,1:2)=stat(spot).Centroid;
            cell.track(old(OldTrajCell,1)).index(k,1)=spot;
            cell.track(old(OldTrajCell,1)).index(k,2)=k;
        else                        % No match, this suitor becomes a new trajectory
            cell.track(end+1).track(k,1:2)=stat(spot).Centroid;
            cell.track(end).index(k,1)=spot;
            cell.track(end).index(k,2)=k;
            distance(v,:)=Inf;
        end
    end
    
    fprintf(repmat('\b',1,n));      % Displays progress of tracking
    msg=sprintf('Tracking... Frame %d/%d. \n',k,Kcount);
    fprintf(msg);
    n=numel(msg);
    
    
% CLEANING UP CELL.TRACK            % Speed up processing by discarding short trajectories
    index=1;
    for u=1:numel(cell.track)
        if size(cell.track(u).track,1)<k && ~isnan(cell.track(u).track(end,1)) 
            cell.track(u).track(k,:)=NaN;       % If a traj didn't find a match, deactivate by capping it with a NaN
        end
        if (nnz(cell.track(u).track)/2)-(nnz(isnan(cell.track(u).track)/2))>=MinimumFrames
            cell.track(index).track=cell.track(u).track;            % If a traj is long enough, keep it
            cell.track(index).index=cell.track(u).index;
            index=index+1;
        elseif k==Kcount || isnan(cell.track(u).track(end,1))       % If a traj is deactivated and not long enough, discard it
        else                                                        % If a traj is still active, keep it
            cell.track(index).track=cell.track(u).track;
            cell.track(index).index=cell.track(u).index;
            index=index+1;
        end
    end
    cell.track(index:end)=[];
end
