function [cell,velocity]=velCalc_v1(C,cell,DarkBackground,Kcount,NmPixel,FrameSeconds,MinimumSeconds,...
    StuckDistance,TempStuckDistance,TempStuckTime,TrackVideo,TrackVideoSeconds,RemoveFluidFlow)

MinimumFrames=MinimumSeconds/FrameSeconds;              % Convert seconds to frames
StuckDistance=StuckDistance/(NmPixel/1000);             % Convert nm to pixels
TempStuckDistance=TempStuckDistance/(NmPixel/1000);     % Convert nm to pixels
TempStuckTime=round(TempStuckTime/FrameSeconds)+1;      % Convert seconds to frames
TrackVideoFrames=round(TrackVideoSeconds/FrameSeconds); % Convert seconds to frames

% CREATING CELL.SIG FOR TRAJECTORIES OF SIGNIFIGANT LENGTH
index=1;
for u=1:numel(cell.track)               	% Trim trajectories of their opening 0's and their NaN caps and place in cell.sig
    if isnan(cell.track(u).track(end,1))
        known = 1;
    else
        known = 0;
    end
    if ((nnz(cell.track(u).track)/2)-known)>=MinimumFrames
        [row,~]=find(cell.track(u).track,1);
        cell.sig(index).track=cell.track(u).track(row:end-known,:);
        cell.sig(index).index=cell.track(u).index(row:end,:);
        index=index+1;
    end
end

%%% CALCULATING VELOCITY
for u=1:numel(cell.sig)
    if StuckDistance          	% Check to see if cell is stuck
        absdispFull=hypot((cell.sig(u).track(1,1)-cell.sig(u).track(end,1)),...
            (cell.sig(u).track(1,2)-cell.sig(u).track(end,2)));
        absdispHalf=hypot((cell.sig(u).track(1,1)-cell.sig(u).track(round(end/2),1)),...
            (cell.sig(u).track(1,2)-cell.sig(u).track(round(end/2),2)));
        if absdispFull<StuckDistance && absdispHalf<StuckDistance
            velocity(u)=NaN;    % If cell moves < a certain distance over length of trajectory, consider it completely stuck.
            continue
        end
    end
    moving=zeros(1,length(cell.sig(u).track)-1);        % moving contains movement between frames.
    for v=1:length(cell.sig(u).track)-1                 % Find distance moved
        ydiff=cell.sig(u).track(v,1)-cell.sig(u).track(v+1,1);
        xdiff=cell.sig(u).track(v,2)-cell.sig(u).track(v+1,2);
        moving(v)=hypot(ydiff,xdiff);
    end
    if TempStuckDistance && TempStuckTime       % Check if temporarily stuck at any point in traj
        count=0;
        for v=1:length(moving)
            if moving(v)<=TempStuckDistance                             % If moved less than TempStuck
                count=count+1;
            elseif moving(v)>TempStuckDistance && count==v-1 && v>1     % If starts stuck for less frames than TempStuckTime
                moving(1:v)=NaN;
                cell.sig(u).track(1:v,:)=NaN;
                count=0;
            elseif moving(v)>TempStuckDistance && count>=TempStuckTime  % If the cell becomes unstuck
                moving(v)=NaN;
                count=0;
            else
                count=0;
            end
            if count==length(moving)                            % If was stuck for entire trajectory
                moving(1:end)=NaN;
                cell.sig(u).track(1:end,:)=NaN;
            elseif v==length(moving) && count                   % If ends stuck for less frames than TempStuckTime
                moving(end-count:end)=NaN;
                cell.sig(u).track(v-count+1:end,:)=NaN;
            elseif count==TempStuckTime && count==v             % If starts stuck for TempStuckTime
                moving(1:v)=NaN;
                cell.sig(u).track(1:v+1,:)=NaN;
            elseif count==TempStuckTime                         % If has been stuck for TempStuckTime
                moving(v-count:v)=NaN;
                cell.sig(u).track(v-count+1:v+1,:)=NaN;
            elseif count>TempStuckTime                          % If is still stuck
                moving(v)=NaN;
                cell.sig(u).track(v+1,:)=NaN;
            end
        end
    end
    goodlength=length(moving)-sum(isnan(moving));     	% Time of movement (in between included frames)
    distance=nansum(moving);                          	% Distance the cell has travelled overall
    if ~goodlength || ~distance
        velocity(u)=NaN;
    else
        velocity(u)=(distance*NmPixel/1000)/(goodlength*FrameSeconds);      %Velocity.
    end
end
if all(isnan(velocity))
    error('All cells are stuck.')
end

%%% RIDDING OF NOISE FROM FLUID FLOW
drifters=[];
if RemoveFluidFlow                                    % direction contains the direction of movement between
    direction=NaN(numel(cell.sig),Kcount-1);    %   each pair of frames for every traj
    for k=2:Kcount                              % cell.sig has holes in it from velocity calcs above
        for u=1:numel(cell.track)               % cell.track has same traj, just uncut at front and back
            if any(cell.sig(u).index(:,2)==k) && any(cell.sig(u).index(:,2)==k-1) && ~isnan(velocity(u))
                ydiff=cell.track(u).track(k,1)-cell.track(u).track(k-1,1);
                xdiff=cell.track(u).track(k,2)-cell.track(u).track(k-1,2);
                if ydiff && xdiff
                    direction(u,k-1)=atan2d(ydiff,xdiff);   % Finds actual direction of movement
                else                                        % Range of directions is -180 to 180 degrees
                    direction(u,k-1)=NaN;
                end
            end
        end
    end
    direction(direction==-180)=180;                            % 0 degrees = 360 degrees
    round_direction=round(direction/15)*15;                 % Aggregate like angles (15 degree window)
    round_direction(round_direction==-180)=180; 
    [M1,F1]=mode(round_direction);                          % Find mode direction for each pair of frames
    direction2=round_direction;
    for k=1:size(direction,2)
        direction2(round_direction(:,k)==M1(k),k)=NaN;
    end
    [M2,F2]=mode(direction2);               % Find "second mode", or what mode would be if we ignored all mode values
    ModeCheck=abs(M2-M1);                   % Check if second mode is near first mode
    ModeDir=zeros(size(M1));
    for s=1:length(M1)                      % s refers to each pair of frames
        if ModeCheck(s)>180
            ModeCheck(s)=360-ModeCheck(s);
        end
        if ModeCheck(s)>15                          % If not near, use first mode
            ModeDir(s)=M1(s);
        else                                        % If near, mode direction is in between
            ModeDir(s)=((abs(M1(s))*F1(s))+(abs(M2(s))*F2(s)))/(F1(s)+F2(s));
            if M1(s)<0 || M2(s)<0
                ModeDir(s)=-ModeDir(s);
            end
        end
    end
    
    
    for s=1:size(direction,2)                       % For each frame, find out which traj go in mode direction
        anglediffs=abs(direction(:,s)-ModeDir(s));
        for v=1:length(anglediffs)
            if anglediffs(v)>180
                anglediffs(v)=360-anglediffs(v);
            end
        end
        potentials=find(anglediffs<15);             % Cutoff for same direction is less than 15 degrees
        driftersperframe(1:length(potentials),s)=potentials;
        antipotentials=find(anglediffs>90);         % Cutoff for opposite direction is greater than 90 degrees
        antidriftersperframe(1:length(antipotentials),s)=antipotentials;
    end
    drifters=unique(driftersperframe)'; % drifters = every traj that goes in mode direction between frames
    drifters(1)=[];
    vees=zeros(size(drifters));         % vees = number of times each traj in drifters goes in mode direction
    for v=1:length(drifters)
        vees(v)=nnz(driftersperframe==drifters(v));
    end
    antivees=zeros(size(drifters));     % antivees = number of times each traj in drifters goes opposite of mode direction
    for v=1:length(drifters)
        antivees(v)=nnz(antidriftersperframe==drifters(v));
    end
    
    
% FIND FLUID VELOCITY       % Finds average fluid velocity based on the velocity of the drifters with the highest vees values
    STDcutoff=1;
    VEEScutoff=1;                   % Cutoff for vees, increases by one with each loop
    while true
        topdrifters=drifters;       % topdrifters are drifters with vees higher than VEEScutoff, fewer as VEEScutoff grows
        topdrifters(vees<=VEEScutoff)=[];
        if std(velocity(topdrifters))<STDcutoff    	% Once the standard dev of the topdrifters' velocities is below STDcutoff
            break                                   %  the average topdrifter velocity is accepted as average fluid velocity
        end
        if isnan(std(velocity(topdrifters)))       	% If the standard dev of the topdrifters' velocities never falls below
            VEEScutoff=1;                           %  STDcutoff, STDcutoff is increased and the process is restarted
            STDcutoff=STDcutoff+0.25;
            if STDcutoff>50
                error('Problem with using RemoveFluidFlow for this video. Please deactivate.')
            end
            continue
        end
        VEEScutoff=VEEScutoff+1;
    end
    avgQ=mean(velocity(topdrifters));
    
% FINDING AFFECTED TRAJECTORIES    
    for v=1:length(drifters)   	% Eliminates drifters based on velocity
        if velocity(drifters(v))>(avgQ+(STDcutoff*2.5)) || velocity(drifters(v))<(avgQ-(STDcutoff*2.5)) 
            drifters(v)=0;
        end
    end
    vees(drifters==0)=[];
    antivees(drifters==0)=[];
    drifters(drifters==0)=[];
    
    for v=1:length(drifters)   	% Eliminates drifters if ratio of vees/antivees is too low and/or velocity isn't close to avg
        if ((((vees(v)/antivees(v)))/abs(velocity(drifters(v))-avgQ)<1)...      
                || (vees(v)/antivees(v))<1) && ~((vees(v)/antivees(v))>=1.5)
            drifters(v)=0;
        end
    end
    vees(drifters==0)=[];
    antivees(drifters==0)=[];
    drifters(drifters==0)=[];
    
    velocity(drifters)=NaN;
end


% VELOCITY HISTOGRAM
figure
edges=0:ceil(max(velocity));
histogram(velocity,edges);
xlabel('Velocity (\mum/sec)')
ylabel('# of Trajectories')
title('Trajectory Velocities')
saveas(gcf,'VelocityHistogram.jpg')


%%% MOVIE
if TrackVideo
    figure
    circle(Kcount).Y=[];                % These structures will contain the centroid data for display
    circle(Kcount).X=[];
    
    for k=1:Kcount
        for u=1:numel(cell.track)
            [row,~]=find(cell.track(u).track,1);        % Find where each track starts
            if TrackVideoFrames && (length(cell.track(u).index)-row)+1>=TrackVideoFrames    % This option only shows accepted
                if length(cell.track(u).track)<k || k-TrackVideoFrames<row...               %  traj with the same length
                        || isnan(cell.track(u).track(k,1)) || any(drifters==u)
                else
                    circle(k).Y=[circle(k).Y;cell.track(u).track(k-TrackVideoFrames+1:k,1);NaN];
                    circle(k).X=[circle(k).X;cell.track(u).track(k-TrackVideoFrames+1:k,2);NaN];
                end
            else                                        % This option shows entirety of accepted traj
                if length(cell.track(u).track)<k || isnan(cell.track(u).track(k,1)) || any(drifters==u)
                else
                    circle(k).Y=[circle(k).Y;cell.track(u).track(row:k,1);NaN];
                    circle(k).X=[circle(k).X;cell.track(u).track(row:k,2);NaN];
                end
            end
        end
    end
    if DarkBackground                	% If video has dark background, cell trail will be white. And vice versa.
        color='w';
    else
        color='k';
    end
    if Kcount==size(C,3)                % If input vid is grayscale
        for k=1:Kcount
            imshow(C(:,:,k));
            line(circle(k).X,circle(k).Y,'Color',color)
            title(['Frame ' num2str(k)])
            Mov(k)=getframe(gcf);
            hold on
        end
    else                            	% If input vid is RGB
        for k=1:Kcount
            imshow(C(:,:,k*3-2:k*3));
            line(circle(k).X,circle(k).Y,'Color',color)
            title(['Frame ' num2str(k)])
            Mov(k)=getframe(gcf);
            hold on
        end
    end
    if TrackVideoFrames
        vidtitle=['TumbleScore(' num2str(TrackVideoFrames) 'sec).mp4'];
    else
        vidtitle=['TumbleScore.mp4'];
    end
    movies=VideoWriter(vidtitle,'MPEG-4');
    movies.FrameRate=(1/FrameSeconds);
    movies.Quality=100;
    open(movies)
    writeVideo(movies,Mov)
end


%%% CREATING CELL.REAL FOR ACCEPTED TRAJECTORIES
index=0;
for u=1:numel(cell.sig)
    if ~isnan(velocity(u)) && velocity(u)>0
        index=index+1;
        cell.real(index).track=cell.sig(u).track;
        cell.real(index).index=cell.sig(u).index;
    end
end
velocity(isnan(velocity))=[];

if numel(cell.real)<10
    fprintf('Caution: Only %i trajectories found. /n',numel(cell.real));
elseif ~isfield(cell,'real')
    error('All cells are stuck.')
end